@@ -1,5 +1,4 @@
-/* liblouis Braille Translation and Back-Translation 
-Library
+/* liblouis Braille Translation and Back-Translation Library
 
    Based on the Linux screenreader BRLTTY, copyright (C) 1999-2006 by
    The BRLTTY Team
@@ -26,7 +25,6 @@ Library
    the Free Software Foundation, 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 
-   Maintained by John J. Boyer john.boyer@jjb-software.com
    */
 
 #include <stdio.h>
@@ -47,9 +45,9 @@ static widechar *currentOutput;
 static unsigned char *typebuf = NULL;
 static int *srcMapping = NULL;
 static char *spacebuf;
-static int backTranslateString (void);
-static int makeCorrections (void);
-static int translatePass (void);
+static int backTranslateString ();
+static int makeCorrections ();
+static int translatePass ();
 
 static int *outputPositions;
 static int *inputPositions;
@@ -57,31 +55,35 @@ static int cursorPosition;
 static int cursorStatus;
 
 int EXPORT_CALL
-lou_backTranslateString (const char *trantab, const widechar
+lou_backTranslateString (const char *tableList, const widechar
 			 * inbuf,
-			 int *inlen, widechar * outbuf, int *outlen, char
+			 int *inlen, widechar * outbuf, int *outlen, 
+			 formtype
 			 *typeform, char *spacing, int modex)
 {
-  return lou_backTranslate (trantab, inbuf, inlen, outbuf, outlen,
+  return lou_backTranslate (tableList, inbuf, inlen, outbuf, outlen,
 			    typeform, spacing, NULL, NULL, NULL, modex);
 }
 
 int EXPORT_CALL
-lou_backTranslate (const char *trantab, const
+lou_backTranslate (const char *tableList, const
 		   widechar
 		   * inbuf,
 		   int *inlen, widechar * outbuf, int *outlen,
-		   char *typeform, char *spacing, int
+		   formtype *typeform, char *spacing, int
 		   *outputPos, int *inputPos, int *cursorPos, int modex)
 {
   int k;
   int goodTrans = 1;
+  if (tableList == NULL || inbuf == NULL || inlen == NULL || outbuf ==
+      NULL || outlen == NULL)
+    return 0;
   if ((modex & otherTrans))
-    return other_backTranslate (trantab, inbuf,
+    return other_backTranslate (tableList, inbuf,
 				inlen, outbuf, outlen,
 				typeform, spacing, outputPos, inputPos,
 				cursorPos, modex);
-  table = lou_getTable (trantab);
+  table = lou_getTable (tableList);
   if (table == NULL)
     return 0;
   srcmax = 0;
@@ -374,7 +376,7 @@ static widechar before, after;
 static TranslationTableCharacterAttributes beforeAttributes;
 static TranslationTableCharacterAttributes afterAttributes;
 static void
-back_setBefore (void)
+back_setBefore ()
 {
   before = (dest == 0) ? ' ' : currentOutput[dest - 1];
   beforeAttributes = (back_findCharOrDots (before, 0))->attributes;
@@ -389,7 +391,7 @@ back_setAfter (int length)
 
 
 static int
-isBegWord (void)
+isBegWord ()
 {
 /*See if this is really the beginning of a word. Look at what has 
 * already been translated. */
@@ -409,7 +411,7 @@ isBegWord (void)
 }
 
 static int
-isEndWord (void)
+isEndWord ()
 {
 /*See if this is really the end of a word. */
   int k;
@@ -443,7 +445,6 @@ isEndWord (void)
     }
   return 1;
 }
-
 static int
 findBrailleIndicatorRule (TranslationTableOffset offset)
 {
@@ -459,7 +460,7 @@ static int doingMultind = 0;
 static const TranslationTableRule *multindRule;
 
 static int
-handleMultind (void)
+handleMultind ()
 {
 /*Handle multille braille indicators*/
   int found = 0;
@@ -532,11 +533,11 @@ static int endMatch;
 static int startReplace;
 static int endReplace;
 
-static int back_passDoTest (void);
-static int back_passDoAction (void);
+static int back_passDoTest ();
+static int back_passDoAction ();
 
 static int
-findAttribOrSwapRules (void)
+findAttribOrSwapRules ()
 {
   TranslationTableOffset ruleOffset;
   if (src == previousSrc)
@@ -555,7 +556,7 @@ findAttribOrSwapRules (void)
 }
 
 static void
-back_selectRule (void)
+back_selectRule ()
 {
 /*check for valid back-translations */
   int length = srcmax - src;
@@ -587,22 +588,6 @@ back_selectRule (void)
 	    break;
 	  length = 1;
 	  ruleOffset = dots->otherRules;
-	  if (itsANumber)
-	    {
-	      while (ruleOffset)
-		{
-		  currentRule =
-		    (TranslationTableRule *) & table->ruleArea[ruleOffset];
-		  if (currentRule->opcode == CTO_LitDigit)
-		    {
-		      currentOpcode = currentRule->opcode;
-		      currentDotslen = currentRule->dotslen;
-		      return;
-		    }
-		  ruleOffset = currentRule->dotsnext;
-		}
-	      ruleOffset = dots->otherRules;
-	    }
 	  break;
 	case 2:		/*No rule found */
 	  currentRule = &pseudoRule;
@@ -641,7 +626,6 @@ back_selectRule (void)
 		    case CTO_Punctuation:
 		    case CTO_Math:
 		    case CTO_Sign:
-		    case CTO_Always:
 		    case CTO_ExactDots:
 		    case CTO_NoCross:
 		    case CTO_Repeated:
@@ -679,7 +663,7 @@ back_selectRule (void)
 		    case CTO_LargeSign:
 		      return;
 		    case CTO_WholeWord:
-		      if (itsALetter)
+		      if (itsALetter || itsANumber)
 			break;
 		    case CTO_Contraction:
 		      if ((beforeAttributes & (CTC_Space | CTC_Punctuation))
@@ -700,10 +684,7 @@ back_selectRule (void)
 			return;
 		      break;
 		    case CTO_SuffixableWord:
-		      if ((beforeAttributes & (CTC_Space | CTC_Punctuation))
-			  &&
-			  ((afterAttributes & (CTC_Space | CTC_Letter))
-			   || isEndWord ()))
+		      if (beforeAttributes & (CTC_Space | CTC_Punctuation))
 			return;
 		      break;
 		    case CTO_PrefixableWord:
@@ -724,7 +705,9 @@ back_selectRule (void)
 			return;
 		      break;
 		    case CTO_PartWord:
-		      if (beforeAttributes & CTC_Letter || !isEndWord ())
+		      if (!(beforeAttributes & 
+		      CTC_LitDigit) && (beforeAttributes & CTC_Letter || 
+		      !isEndWord ()))
 			return;
 		      break;
 		    case CTO_MidWord:
@@ -766,6 +749,12 @@ back_selectRule (void)
 		      if (isEndWord ())
 			return;
 		      break;
+		    case CTO_Always:
+		    if ((beforeAttributes & CTC_LitDigit) && 
+		    (afterAttributes & CTC_LitDigit) && 
+		    currentRule->charslen > 1)
+		    break;
+		    return;
 		    default:
 		      break;
 		    }
@@ -919,7 +908,7 @@ putCharacters (const widechar * characters, int count)
 }
 
 static int
-insertSpace (void)
+insertSpace ()
 {
   widechar c = ' ';
   if (!back_updatePositions (&c, 1, 1))
@@ -944,7 +933,7 @@ compareChars (const widechar * address1, const widechar * address2, int
 }
 
 static int
-makeCorrections (void)
+makeCorrections ()
 {
   int k;
   if (!table->corrections)
@@ -1031,7 +1020,7 @@ failure:
 }
 
 static int
-backTranslateString (void)
+backTranslateString ()
 {
 /*Back translation */
   int srcword = 0;
@@ -1048,7 +1037,6 @@ backTranslateString (void)
       switch (currentOpcode)
 	{
 	case CTO_Hyphen:
-	  if (isEndWord ())
 	    itsANumber = 0;
 	  break;
 	case CTO_LargeSign:
@@ -1150,7 +1138,7 @@ backTranslateString (void)
 	  itsANumber = 0;
 	  goto insertChars;
 	case CTO_Space:
-	  itsANumber = allUpper = nextUpper = 0;
+	  itsALetter = itsANumber = allUpper = nextUpper = 0;
 	default:
 	insertChars:
 	  if (currentRule->charslen)
@@ -1215,7 +1203,7 @@ failure:
 /*Multipass translation*/
 
 static int
-matchcurrentInput (void)
+matchcurrentInput ()
 {
   int k;
   int kk = passSrc;
@@ -1226,7 +1214,7 @@ matchcurrentInput (void)
 }
 
 static int
-back_swapTest (void)
+back_swapTest ()
 {
   int curLen;
   int curTest;
@@ -1462,7 +1450,7 @@ back_passDoTest ()
 }
 
 static int
-back_passDoAction (void)
+back_passDoAction ()
 {
   int k;
   if ((dest + startReplace - startMatch) > destmax)
@@ -1528,7 +1516,7 @@ back_passDoAction (void)
 }
 
 static int
-checkDots (void)
+checkDots ()
 {
   int k;
   int kk = src;
@@ -1539,7 +1527,7 @@ checkDots (void)
 }
 
 static void
-for_passSelectRule (void)
+for_passSelectRule ()
 {
   int length = srcmax - src;
   const TranslationTableCharacter *dots;
@@ -1613,7 +1601,7 @@ for_passSelectRule (void)
 }
 
 static int
-translatePass (void)
+translatePass ()
 {
   int k;
   previousOpcode = CTO_None;
