@@ -1,5 +1,4 @@
-/* liblouis Braille Translation and Back-Translation 
-Library
+/* liblouis Braille Translation and Back-Translation Library
 
    Based on the Linux screenreader BRLTTY, copyright (C) 1999-2006 by
    The BRLTTY Team
@@ -7,26 +6,25 @@ Library
    Copyright (C) 2004, 2005, 2006
    ViewPlus Technologies, Inc. www.viewplus.com
    and
-   JJB Software, Inc. www.jjb-software.com
+   abilitiessoft, Inc. www.abilitiessoft.org
    All rights reserved
 
-   This file is free software; you can redistribute it and/or modify it
-   under the terms of the Lesser or Library GNU General Public License 
-   as published by the
-   Free Software Foundation; either version 3, or (at your option) any
-   later version.
+   This file is part of Liblouis.
 
-   This file is distributed in the hope that it will be useful, but
+   Liblouis is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation, either version 3 of the
+   License, or (at your option) any later version.
+
+   Liblouis is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
-   Library GNU General Public License for more details.
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+   Lesser General Public License for more details.
 
-   You should have received a copy of the Library GNU General Public 
-   License along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+   You should have received a copy of the GNU Lesser General Public
+   License along with Liblouis. If not, see
+   <http://www.gnu.org/licenses/>.
 
-   Maintained by John J. Boyer john.boyer@jjb-software.com
    */
 
 #include <stdio.h>
@@ -45,7 +43,8 @@ static int compbrlEnd = 0;
 int EXPORT_CALL
 lou_translateString (const char *tableList, const widechar
 		     * inbufx,
-		     int *inlen, widechar * outbuf, int *outlen, char
+		     int *inlen, widechar * outbuf, int *outlen, 
+		     formtype
 		     *typeform, char *spacing, int mode)
 {
   return
@@ -57,7 +56,7 @@ int EXPORT_CALL
 lou_translate (const char *tableList, const widechar
 	       * inbufx,
 	       int *inlen, widechar * outbuf, int *outlen,
-	       char *typeform, char *spacing, int *outputPos,
+	       formtype *typeform, char *spacing, int *outputPos,
 	       int *inputPos, int *cursorPos, int modex)
 {
   return trace_translate (tableList, inbufx, inlen, outbuf, outlen,
@@ -68,7 +67,7 @@ lou_translate (const char *tableList, const widechar
 int
 trace_translate (const char *tableList, const widechar * inbufx,
 		 int *inlen, widechar * outbuf, int *outlen,
-		 char *typeform, char *spacing, int *outputPos,
+		 formtype *typeform, char *spacing, int *outputPos,
 		 int *inputPos, int *cursorPos,
 		 const TranslationTableRule ** rules, int *rulesLen,
 		 int modex)
@@ -78,6 +77,8 @@ trace_translate (const char *tableList, const widechar * inbufx,
   if (tableList == NULL || inbufx == NULL || inlen == NULL || outbuf ==
       NULL || outlen == NULL)
     return 0;
+  logMessage(LOG_DEBUG, "Performing translation: tableList=%s, inlen=%d", tableList, *inlen);
+  logWidecharBuf(LOG_DEBUG, "Inbuf=", inbufx, *inlen);
   if ((modex & otherTrans))
     return other_translate (tableList, inbufx,
 			    inlen, outbuf, outlen,
@@ -246,7 +247,7 @@ trace_translate (const char *tableList, const widechar * inbufx,
       *inlen = realInlen;
       *outlen = dest;
       if (inputPositions != NULL)
-	memcpy (inputPositions, srcMapping, destmax * sizeof (int));
+	memcpy (inputPositions, srcMapping, dest * sizeof (int));
       if (outputPos != NULL)
 	{
 	  int lastpos = 0;
@@ -271,6 +272,8 @@ trace_translate (const char *tableList, const widechar * inbufx,
     }
   if (rulesLen != NULL)
     *rulesLen = appliedRulesCount;
+  logMessage(LOG_DEBUG, "Translation complete: outlen=%d", *outlen);
+  logWidecharBuf(LOG_DEBUG, "Outbuf=", (const widechar *)outbuf, *outlen);
   return goodTrans;
 }
 
@@ -278,7 +281,7 @@ int EXPORT_CALL
 lou_translatePrehyphenated (const char *tableList,
 			    const widechar * inbufx, int *inlen,
 			    widechar * outbuf, int *outlen,
-			    char *typeform, char *spacing,
+			    formtype *typeform, char *spacing,
 			    int *outputPos, int *inputPos, int *cursorPos,
 			    char *inputHyphens, char *outputHyphens,
 			    int modex)
@@ -328,8 +331,6 @@ static TranslationTableOpcode indicOpcode;
 static const TranslationTableRule *indicRule;
 static int dontContract = 0;
 
-static int doCompbrl ();
-
 static int
 hyphenate (const widechar * word, int wordSize, char *hyphens)
 {
@@ -345,68 +346,69 @@ hyphenate (const widechar * word, int wordSize, char *hyphens)
   int patternOffset;
   if (!table->hyphenStatesArray || (wordSize + 3) > MAXSTRING)
     return 0;
-  prepWord = (widechar *) calloc(wordSize+3, sizeof(widechar));
+  prepWord = (widechar *) calloc (wordSize + 3, sizeof (widechar));
   /* prepWord is of the format ".hello."
-  * hyphens is the length of the word "hello" "00000" */
+   * hyphens is the length of the word "hello" "00000" */
   prepWord[0] = '.';
   for (i = 0; i < wordSize; i++)
     {
-      prepWord[i+1] = (findCharOrDots (word[i], 0))->lowercase;
+      prepWord[i + 1] = (findCharOrDots (word[i], 0))->lowercase;
       hyphens[i] = '0';
     }
-  prepWord[wordSize+1] = '.';
+  prepWord[wordSize + 1] = '.';
 
   /* now, run the finite state machine */
   stateNum = 0;
 
   // we need to walk all of ".hello."
-  for (i = 0; i < wordSize+2; i++)
+  for (i = 0; i < wordSize + 2; i++)
     {
       ch = prepWord[i];
       while (1)
-        {
-          if (stateNum == 0xffff)
-            {
-              stateNum = 0;
-              goto nextLetter;
-            }
-          currentState = &statesArray[stateNum];
-          if (currentState->trans.offset)
-            {
-              transitionsArray = (HyphenationTrans *) &
-                table->ruleArea[currentState->trans.offset];
-              for (k = 0; k < currentState->numTrans; k++)
-                {
-                  if (transitionsArray[k].ch == ch)
-                    {
-                      stateNum = transitionsArray[k].newState;
-                      goto stateFound;
-                    }
-                }
-            }
-            stateNum = currentState->fallbackState;
-        }
+	{
+	  if (stateNum == 0xffff)
+	    {
+	      stateNum = 0;
+	      goto nextLetter;
+	    }
+	  currentState = &statesArray[stateNum];
+	  if (currentState->trans.offset)
+	    {
+	      transitionsArray = (HyphenationTrans *) &
+		table->ruleArea[currentState->trans.offset];
+	      for (k = 0; k < currentState->numTrans; k++)
+		{
+		  if (transitionsArray[k].ch == ch)
+		    {
+		      stateNum = transitionsArray[k].newState;
+		      goto stateFound;
+		    }
+		}
+	    }
+	  stateNum = currentState->fallbackState;
+	}
     stateFound:
       currentState = &statesArray[stateNum];
       if (currentState->hyphenPattern)
-        {
-          hyphenPattern = (char *) &table->ruleArea[currentState->hyphenPattern];
-          patternOffset = i+1 - strlen(hyphenPattern);
+	{
+	  hyphenPattern =
+	    (char *) &table->ruleArea[currentState->hyphenPattern];
+	  patternOffset = i + 1 - strlen (hyphenPattern);
 
-          /* Need to ensure that we don't overrun hyphens,
-          * in some cases hyphenPattern is longer than the remaining letters,
-          * and if we write out all of it we would have overshot our buffer. */
-          limit = MIN(strlen(hyphenPattern), wordSize-patternOffset);
-          for (k = 0; k<limit; k++)
-            {
-              if (hyphens[patternOffset + k] < hyphenPattern[k])
-                hyphens[patternOffset + k] = hyphenPattern[k];
-            }
-        }
+	  /* Need to ensure that we don't overrun hyphens,
+	   * in some cases hyphenPattern is longer than the remaining letters,
+	   * and if we write out all of it we would have overshot our buffer. */
+	  limit = MIN (strlen (hyphenPattern), wordSize - patternOffset);
+	  for (k = 0; k < limit; k++)
+	    {
+	      if (hyphens[patternOffset + k] < hyphenPattern[k])
+		hyphens[patternOffset + k] = hyphenPattern[k];
+	    }
+	}
     nextLetter:;
     }
   hyphens[wordSize] = 0;
-  free(prepWord);
+  free (prepWord);
   return 1;
 }
 
@@ -481,45 +483,46 @@ for_updatePositions (const widechar * outChars, int inLength, int outLength)
 static int
 syllableBreak ()
 {
-  int wordStart=0;
-  int wordEnd=0;
-  int wordSize=0;
-  int k=0;
+  int wordStart = 0;
+  int wordEnd = 0;
+  int wordSize = 0;
+  int k = 0;
   char *hyphens = NULL;
   for (wordStart = src; wordStart >= 0; wordStart--)
     if (!((findCharOrDots (currentInput[wordStart], 0))->attributes &
-          CTC_Letter))
+	  CTC_Letter))
       {
-        wordStart++;
-        break;
+	wordStart++;
+	break;
       }
   if (wordStart < 0)
     wordStart = 0;
   for (wordEnd = src; wordEnd < srcmax; wordEnd++)
     if (!((findCharOrDots (currentInput[wordEnd], 0))->attributes &
-          CTC_Letter))
+	  CTC_Letter))
       {
-        wordEnd--;
-        break;
+	wordEnd--;
+	break;
       }
-  if (wordEnd == srcmax) wordEnd--;
+  if (wordEnd == srcmax)
+    wordEnd--;
   /* At this stage wordStart is the 0 based index of the first letter in the word,
-  * wordEnd is the 0 based index of the last letter in the word.
-  * example: "hello" wordstart=0, wordEnd=4. */
-  wordSize=wordEnd-wordStart+1;
-  hyphens = (char *) calloc(wordSize+1, sizeof(char));
+   * wordEnd is the 0 based index of the last letter in the word.
+   * example: "hello" wordstart=0, wordEnd=4. */
+  wordSize = wordEnd - wordStart + 1;
+  hyphens = (char *) calloc (wordSize + 1, sizeof (char));
   if (!hyphenate (&currentInput[wordStart], wordSize, hyphens))
     {
-      free(hyphens);
+      free (hyphens);
       return 0;
     }
-  for (k = src - wordStart+1; k < (src - wordStart + transCharslen); k++) 
+  for (k = src - wordStart + 1; k < (src - wordStart + transCharslen); k++)
     if (hyphens[k] & 1)
       {
-        free(hyphens);
-        return 1;
-    }
-  free(hyphens);
+	free (hyphens);
+	return 1;
+      }
+  free (hyphens);
   return 0;
 }
 
@@ -563,8 +566,8 @@ brailleIndicatorDefined (TranslationTableOffset offset)
   return 1;
 }
 
-static typeforms prevType = plain_text;
-static typeforms curType = plain_text;
+static int prevType = plain_text;
+static int curType = plain_text;
 
 typedef enum
 {
@@ -677,22 +680,8 @@ validMatch ()
   TranslationTableCharacterAttributes prevAttr = 0;
   int k;
   int kk = 0;
-  unsigned short mask = 0;
   if (!transCharslen)
     return 0;
-  switch (transOpcode)
-    {
-    case CTO_WholeWord:
-    case CTO_PrefixableWord:
-    case CTO_SuffixableWord:
-    case CTO_JoinableWord:
-    case CTO_LowWord:
-      mask = EMPHASIS | capsemph;
-      break;
-    default:
-      mask = EMPHASIS | SYLLABLEMARKS | INTERNALMARKS | capsemph;
-      break;
-    }
   for (k = src; k < src + transCharslen; k++)
     {
       if (currentInput[k] == ENDSEGMENT)
@@ -709,7 +698,7 @@ validMatch ()
       if ((currentInputChar->lowercase != ruleChar->lowercase))
 	return 0;
       if (typebuf != NULL && (typebuf[src] & capsemph) == 0 &&
-	  (typebuf[k] & mask) != (typebuf[src] & mask))
+	  (typebuf[k] | typebuf[src]) != (typebuf[src]))
 	return 0;
       if (currentInputChar->attributes != CTC_Letter)
 	{
@@ -788,7 +777,7 @@ endEmphasis (const TranslationTableOffset * offset)
       brailleIndicatorDefined (offset[singleLetter]))
     return 0;
   else
-    if ((finishEmphasis || (src < srcmax && ((findCharOrDots
+    if ((finishEmphasis || src == srcmax - 1 || (src < srcmax && ((findCharOrDots
 					      (currentInput[src + 1],
 					       0))->attributes &
 					     CTC_Letter)))
@@ -834,7 +823,7 @@ insertBrailleIndicators (int finish)
 			  currentOutput[dest - 1] == B16))
 	dest--;
       finishEmphasis = 1;
-      prevType = prevPrevType;
+      prevType = prevPrevType = prevTypeform & EMPHASIS;
       curType = plain_text;
       checkWhat = checkEndTypeform;
     }
@@ -844,7 +833,7 @@ insertBrailleIndicators (int finish)
 	return 1;
       if (src != prevSrc)
 	{
-	  if (haveEmphasis && src < srcmax)
+	  if (haveEmphasis && src < srcmax - 1)
 	    nextType = typebuf[src + 1] & EMPHASIS;
 	  else
 	    nextType = plain_text;
@@ -1000,19 +989,17 @@ insertBrailleIndicators (int finish)
 		prevType = 0;
 		break;
 	      }
-	  if (!prevType)
+	  if (prevType == plain_text)
 	    {
 	      checkWhat = checkBeginTypeform;
 	      prevTypeform = typebuf[src] & EMPHASIS;
 	    }
 	  break;
 	case checkNumber:
-	  if (brailleIndicatorDefined
-	      (table->numberSign) &&
-	      checkAttr (currentInput[src], CTC_Digit, 0) &&
-	      (prevTransOpcode == CTO_ExactDots
-	       || !(beforeAttributes & CTC_Digit))
-	      && prevTransOpcode != CTO_MidNum)
+	  if (brailleIndicatorDefined(table->numberSign) &&
+	      checkAttr_safe(currentInput, src, CTC_Digit, 0) &&
+	      (prevTransOpcode == CTO_ExactDots || !(beforeAttributes & CTC_Digit)) &&
+	      prevTransOpcode != CTO_MidNum)
 	    {
 	      ok = 1;
 	      checkWhat = checkNothing;
@@ -1033,10 +1020,10 @@ insertBrailleIndicators (int finish)
 	      checkWhat = checkBeginMultCaps;
 	      break;
 	    }
-	  if ((checkAttr (currentInput[src], CTC_Letter, 0)
-	       && !(beforeAttributes & CTC_Letter))
-	      && (!checkAttr (currentInput[src + 1], CTC_Letter, 0)
-		  || (beforeAttributes & CTC_Digit)))
+	  if ((checkAttr_safe(currentInput, src, CTC_Letter, 0) &&
+	       !(beforeAttributes & CTC_Letter)) &&
+	      (!checkAttr_safe(currentInput, src + 1, CTC_Letter, 0) ||
+	       (beforeAttributes & CTC_Digit)))
 	    {
 	      ok = 1;
 	      if (src > 0)
@@ -1076,9 +1063,9 @@ insertBrailleIndicators (int finish)
 	  break;
 	case checkEndMultCaps:
 	  if (brailleIndicatorDefined (table->endCapitalSign) &&
-	      (prevPrevAttr & CTC_UpperCase)
-	      && (beforeAttributes & CTC_UpperCase)
-	      && checkAttr (currentInput[src], CTC_LowerCase, 0))
+	      (prevPrevAttr & CTC_UpperCase) &&
+	      (beforeAttributes & CTC_UpperCase) &&
+	      checkAttr_safe(currentInput, src, CTC_LowerCase, 0))
 	    {
 	      ok = 1;
 	      if (table->capsNoCont)
@@ -1087,8 +1074,9 @@ insertBrailleIndicators (int finish)
 	  checkWhat = checkNothing;
 	  break;
 	case checkSingleCap:
-	  if (brailleIndicatorDefined (table->capitalSign) && src < srcmax
-	      && checkAttr (currentInput[src], CTC_UpperCase, 0) &&
+	  if (brailleIndicatorDefined (table->capitalSign) &&
+	      src < srcmax &&
+	      checkAttr_safe(currentInput, src, CTC_UpperCase, 0) &&
 	      (!(beforeAttributes & CTC_UpperCase) ||
 	       table->beginCapitalSign == 0))
 	    {
@@ -1362,7 +1350,9 @@ for_selectRule ()
 		  case CTO_ExactDots:
 		    return;
 		  case CTO_NoCross:
-		    if (syllableBreak ())
+if (dontContract || (mode & noContractions))
+		      break;
+		    		    if (syllableBreak ())
 		      break;
 		    return;
 		  case CTO_Context:
@@ -1555,6 +1545,7 @@ undefinedCharacter (widechar c)
 /*Display an undefined character in the output buffer*/
   int k;
   char *display;
+  widechar displayDots[20];
   if (table->undefined)
     {
       TranslationTableRule *transRule = (TranslationTableRule *)
@@ -1566,16 +1557,10 @@ undefinedCharacter (widechar c)
       return 1;
     }
   display = showString (&c, 1);
-  if ((dest + strlen (display)) > destmax)
-    return 0;
-  if (outputPositions != NULL)
-    outputPositions[prevSrcMapping[src]] = dest;
   for (k = 0; k < strlen (display); k++)
-    {
-      if (inputPositions != NULL)
-	srcMapping[dest] = prevSrcMapping[src];
-      currentOutput[dest++] = getDotsForChar (display[k]);
-    }
+    displayDots[k] = getDotsForChar (display[k]);
+  if (!for_updatePositions (displayDots, 1, strlen(display)))
+    return 0;
   return 1;
 }
 
@@ -1622,7 +1607,7 @@ static int
 doCompbrl ()
 {
 /*Handle strings containing substrings defined by the compbrl opcode*/
-  int stringEnd;
+  int stringStart, stringEnd;
   if (checkAttr (currentInput[src], CTC_Space, 0))
     return 1;
   if (destword)
@@ -1635,10 +1620,14 @@ doCompbrl ()
       src = 0;
       dest = 0;
     }
+  for (stringStart = src; stringStart >= 0; stringStart--)
+    if (checkAttr (currentInput[stringStart], CTC_Space, 0))
+      break;
+  stringStart++;
   for (stringEnd = src; stringEnd < srcmax; stringEnd++)
     if (checkAttr (currentInput[stringEnd], CTC_Space, 0))
       break;
-  return (doCompTrans (src, stringEnd));
+  return (doCompTrans (stringStart, stringEnd));
 }
 
 static int
@@ -1665,6 +1654,7 @@ static int
 doCompTrans (int start, int end)
 {
   int k;
+  int haveEndsegment = 0;
   if (cursorStatus != 2 && brailleIndicatorDefined (table->begComp))
     if (!for_updatePositions
 	(&indicRule->charsdots[0], 0, indicRule->dotslen))
@@ -1672,6 +1662,11 @@ doCompTrans (int start, int end)
   for (k = start; k < end; k++)
     {
       TranslationTableOffset compdots = 0;
+      if (currentInput[k] == ENDSEGMENT)
+	{
+	  haveEndsegment = 1;
+	  continue;
+	}
       src = k;
       if (currentInput[k] < 256)
 	compdots = table->compdotsPattern[currentInput[k]];
@@ -1691,6 +1686,12 @@ doCompTrans (int start, int end)
 	(&indicRule->charsdots[0], 0, indicRule->dotslen))
       return 0;
   src = end;
+  if (haveEndsegment)
+    {
+      widechar endSegment = ENDSEGMENT;
+      if (!for_updatePositions (&endSegment, 0, 1))
+	return 0;
+    }
   return 1;
 }
 
@@ -1741,6 +1742,7 @@ markSyllables ()
 	      if (!(length >= 2))
 		break;
 	      makeHash = (unsigned long int) character->lowercase << 8;
+		  //memory overflow when src == srcmax - 1
 	      character2 = findCharOrDots (currentInput[src + 1], 0);
 	      makeHash += (unsigned long int) character2->lowercase;
 	      makeHash %= HASHNUM;
@@ -1792,7 +1794,7 @@ markSyllables ()
 	  if (syllableMarker > 3)
 	    syllableMarker = 1;
 	  currentMark = syllableMarker << 6;
-	  /*The syllable marker is bita 6 and 7 of typebuf. */
+	  /*The syllable marker is bits 6 and 7 of typebuf. */
 	  if ((src + transCharslen) > srcmax)
 	    return 0;
 	  for (k = 0; k < transCharslen; k++)
@@ -1812,7 +1814,7 @@ translateString ()
   int k;
   markSyllables ();
   srcword = 0;
-  destword = 0;			/* last word translated */
+  destword = 0;        		/* last word translated */
   dontContract = 0;
   prevTransOpcode = CTO_None;
   wordsMarked = 0;
@@ -1820,232 +1822,247 @@ translateString ()
   startType = prevSrc = -1;
   src = dest = 0;
   srcIncremented = 1;
-  for (k = 0; k < NUMVAR; k++)
-    passVariables[k] = 0;
+  memset (passVariables, 0, sizeof(int) * NUMVAR);
   if (typebuf && table->capitalSign)
     for (k = 0; k < srcmax; k++)
       if (checkAttr (currentInput[k], CTC_UpperCase, 0))
-	typebuf[k] |= capsemph;
+        typebuf[k] |= capsemph;
   while (src < srcmax)
-    {				/*the main translation loop */
+    {        			/*the main translation loop */
       setBefore ();
       if (!insertBrailleIndicators (0))
-	goto failure;
+        goto failure;
       if (src >= srcmax)
-	break;
+        break;
       if (!insertIndicators ())
-	goto failure;
+        goto failure;
       for_selectRule ();
       if (appliedRules != NULL && appliedRulesCount < maxAppliedRules)
-	appliedRules[appliedRulesCount++] = transRule;
+        appliedRules[appliedRulesCount++] = transRule;
       srcIncremented = 1;
       prevSrc = src;
-      switch (transOpcode)	/*Rules that pre-empt context and swap */
-	{
-	case CTO_CompBrl:
-	case CTO_Literal:
-	  if (!doCompbrl ())
-	    goto failure;
-	  continue;
-	default:
-	  break;
-	}
+      switch (transOpcode)        /*Rules that pre-empt context and swap */
+        {
+        case CTO_CompBrl:
+        case CTO_Literal:
+          if (!doCompbrl ())
+            goto failure;
+          continue;
+        default:
+          break;
+        }
       if (!insertBrailleIndicators (1))
-	goto failure;
+        goto failure;
       if (transOpcode == CTO_Context || findAttribOrSwapRules ())
-	switch (transOpcode)
-	  {
-	  case CTO_Context:
-	    if (appliedRules != NULL && appliedRulesCount < maxAppliedRules)
-	      appliedRules[appliedRulesCount++] = transRule;
-	    if (!passDoAction ())
-	      goto failure;
-	    if (endReplace == src)
-	      srcIncremented = 0;
-	    src = endReplace;
-	    continue;
-	  default:
-	    break;
-	  }
+        switch (transOpcode)
+          {
+          case CTO_Context:
+            if (appliedRules != NULL && appliedRulesCount < maxAppliedRules)
+              appliedRules[appliedRulesCount++] = transRule;
+            if (!passDoAction ())
+              goto failure;
+            if (endReplace == src)
+              srcIncremented = 0;
+            src = endReplace;
+            continue;
+          default:
+            break;
+          }
 
 /*Processing before replacement*/
       switch (transOpcode)
-	{
-	case CTO_EndNum:
-	  if (table->letterSign && checkAttr (currentInput[src],
-					      CTC_Letter, 0))
-	    dest--;
-	  break;
-	case CTO_Repeated:
-	case CTO_Space:
-	  dontContract = 0;
-	  break;
-	case CTO_LargeSign:
-	  if (prevTransOpcode == CTO_LargeSign)
-	    if (dest > 0 && checkAttr (currentOutput[dest - 1], CTC_Space, 1))
-	      dest--;
-	  break;
-	case CTO_DecPoint:
-	  if (table->numberSign)
-	    {
-	      TranslationTableRule *numRule = (TranslationTableRule *)
-		& table->ruleArea[table->numberSign];
-	      if (!for_updatePositions
-		  (&numRule->charsdots[numRule->charslen],
-		   numRule->charslen, numRule->dotslen))
-		goto failure;
-	    }
-	  transOpcode = CTO_MidNum;
-	  break;
-	case CTO_NoCont:
-	  if (!dontContract)
-	    doNocont ();
-	  continue;
-	default:
-	  break;
-	}			/*end of action */
+        {
+        case CTO_EndNum:
+          if (table->letterSign && checkAttr (currentInput[src],
+        				      CTC_Letter, 0))
+            dest--;
+          break;
+        case CTO_Repeated:
+        case CTO_Space:
+          dontContract = 0;
+          break;
+        case CTO_LargeSign:
+          if (prevTransOpcode == CTO_LargeSign)
+          {
+            int hasEndSegment = 0;
+            while (dest > 0 && checkAttr (currentOutput[dest - 1], CTC_Space, 1))
+            {
+              if (currentOutput[dest - 1] == ENDSEGMENT)
+              {
+                hasEndSegment = 1;
+              }
+              dest--;
+            }
+            if (hasEndSegment != 0)
+            {
+              currentOutput[dest] = 0xffff;
+              dest++;
+            }
+          }
+          break;
+        case CTO_DecPoint:
+          if (table->numberSign)
+            {
+              TranslationTableRule *numRule = (TranslationTableRule *)
+        	& table->ruleArea[table->numberSign];
+              if (!for_updatePositions
+        	  (&numRule->charsdots[numRule->charslen],
+        	   numRule->charslen, numRule->dotslen))
+        	goto failure;
+            }
+          transOpcode = CTO_MidNum;
+          break;
+        case CTO_NoCont:
+          if (!dontContract)
+            doNocont ();
+          continue;
+        default:
+          break;
+        }			/*end of action */
 
       /* replacement processing */
       switch (transOpcode)
-	{
-	case CTO_Replace:
-	  src += transCharslen;
-	  if (!putCharacters
-	      (&transRule->charsdots[transCharslen], transRule->dotslen))
-	    goto failure;
-	  break;
-	case CTO_None:
-	  if (!undefinedCharacter (currentInput[src]))
-	    goto failure;
-	  src++;
-	  break;
-	case CTO_UpperCase:
-	  /* Only needs special handling if not within compbrl and
-	   *the table defines a capital sign. */
-	  if (!
-	      (mode & (compbrlAtCursor | compbrlLeftCursor) && src >=
-	       compbrlStart
-	       && src <= compbrlEnd) && (transRule->dotslen == 1
-					 && table->capitalSign))
-	    {
-	      putCharacter (curCharDef->lowercase);
-	      src++;
-	      break;
-	    }
-	default:
-	  if (cursorStatus == 2)
-	    cursorStatus = 1;
-	  else
-	    {
-	      if (transRule->dotslen)
-		{
-		  if (!for_updatePositions
-		      (&transRule->charsdots[transCharslen],
-		       transCharslen, transRule->dotslen))
-		    goto failure;
-		}
-	      else
-		{
-		  for (k = 0; k < transCharslen; k++)
-		    {
-		      if (!putCharacter (currentInput[src]))
-			goto failure;
-		      src++;
-		    }
-		}
-	      if (cursorStatus == 2)
-		cursorStatus = 1;
-	      else if (transRule->dotslen)
-		src += transCharslen;
-	    }
-	  break;
-	}
+        {
+        case CTO_Replace:
+          src += transCharslen;
+          if (!putCharacters
+              (&transRule->charsdots[transCharslen], transRule->dotslen))
+            goto failure;
+          break;
+        case CTO_None:
+          if (!undefinedCharacter (currentInput[src]))
+            goto failure;
+          src++;
+          break;
+        case CTO_UpperCase:
+          /* Only needs special handling if not within compbrl and
+           *the table defines a capital sign. */
+          if (!
+              (mode & (compbrlAtCursor | compbrlLeftCursor) && src >=
+               compbrlStart
+               && src <= compbrlEnd) && (transRule->dotslen == 1
+        				 && table->capitalSign))
+            {
+              putCharacter (curCharDef->lowercase);
+              src++;
+              break;
+            }
+        default:
+          if (cursorStatus == 2)
+            cursorStatus = 1;
+          else
+            {
+              if (transRule->dotslen)
+        	{
+        	  if (!for_updatePositions
+        	      (&transRule->charsdots[transCharslen],
+        	       transCharslen, transRule->dotslen))
+        	    goto failure;
+        	}
+              else
+        	{
+        	  for (k = 0; k < transCharslen; k++)
+        	    {
+        	      if (!putCharacter (currentInput[src]))
+        		goto failure;
+        	      src++;
+        	    }
+        	}
+              if (cursorStatus == 2)
+        	cursorStatus = 1;
+              else if (transRule->dotslen)
+        	src += transCharslen;
+            }
+          break;
+        }
 
       /* processing after replacement */
       switch (transOpcode)
-	{
-	case CTO_Repeated:
-	  {
-	    /* Skip repeated characters. */
-	    int srclim = srcmax - transCharslen;
-	    if (mode & (compbrlAtCursor | compbrlLeftCursor) &&
-		compbrlStart < srclim)
-	      /* Don't skip characters from compbrlStart onwards. */
-	      srclim = compbrlStart - 1;
-	    while ((src <= srclim)
-		   && compareChars (&transRule->charsdots[0],
-				    &currentInput[src], transCharslen, 0))
-	      {
-		/* Map skipped input positions to the previous output position. */
-		if (outputPositions != NULL)
-		  {
-		    int tcc;
-		    for (tcc = 0; tcc < transCharslen; tcc++)
-		      outputPositions[prevSrcMapping[src + tcc]] = dest - 1;
-		  }
-		if (!cursorStatus && src <= cursorPosition
-		    && cursorPosition < src + transCharslen)
-		  {
-		    cursorStatus = 1;
-		    cursorPosition = dest - 1;
-		  }
-		src += transCharslen;
-	      }
-	    break;
-	  }
-	case CTO_RepWord:
-	  {
-	    /* Skip repeated characters. */
-	    int srclim = srcmax - transCharslen;
-	    if (mode & (compbrlAtCursor | compbrlLeftCursor) &&
-		compbrlStart < srclim)
-	      /* Don't skip characters from compbrlStart onwards. */
-	      srclim = compbrlStart - 1;
-	    while ((src <= srclim)
-		   && compareChars (repwordStart,
-				    &currentInput[src], repwordLength, 0))
-	      {
-		/* Map skipped input positions to the previous output position. */
-		if (outputPositions != NULL)
-		  {
-		    int tcc;
-		    for (tcc = 0; tcc < transCharslen; tcc++)
-		      outputPositions[prevSrcMapping[src + tcc]] = dest - 1;
-		  }
-		if (!cursorStatus && src <= cursorPosition
-		    && cursorPosition < src + transCharslen)
-		  {
-		    cursorStatus = 1;
-		    cursorPosition = dest - 1;
-		  }
-		src += repwordLength + transCharslen;
-	      }
-	    src -= transCharslen;
-	    break;
-	  }
-	case CTO_JoinNum:
-	case CTO_JoinableWord:
-	  while ((src < srcmax)
-		 && checkAttr (currentInput[src], CTC_Space, 0))
-	    src++;
-	  break;
-	default:
-	  break;
-	}
+        {
+        case CTO_Repeated:
+          {
+            /* Skip repeated characters. */
+            int srclim = srcmax - transCharslen;
+            if (mode & (compbrlAtCursor | compbrlLeftCursor) &&
+        	compbrlStart < srclim)
+              /* Don't skip characters from compbrlStart onwards. */
+              srclim = compbrlStart - 1;
+            while ((src <= srclim)
+        	   && compareChars (&transRule->charsdots[0],
+        			    &currentInput[src], transCharslen, 0))
+              {
+        	/* Map skipped input positions to the previous output position. */
+        	if (outputPositions != NULL)
+        	  {
+        	    int tcc;
+        	    for (tcc = 0; tcc < transCharslen; tcc++)
+        	      outputPositions[prevSrcMapping[src + tcc]] = dest - 1;
+        	  }
+        	if (!cursorStatus && src <= cursorPosition
+        	    && cursorPosition < src + transCharslen)
+        	  {
+        	    cursorStatus = 1;
+        	    cursorPosition = dest - 1;
+        	  }
+        	src += transCharslen;
+              }
+            break;
+          }
+        case CTO_RepWord:
+          {
+            /* Skip repeated characters. */
+            int srclim = srcmax - transCharslen;
+            if (mode & (compbrlAtCursor | compbrlLeftCursor) &&
+        	compbrlStart < srclim)
+              /* Don't skip characters from compbrlStart onwards. */
+              srclim = compbrlStart - 1;
+            while ((src <= srclim)
+        	   && compareChars (repwordStart,
+        			    &currentInput[src], repwordLength, 0))
+              {
+        	/* Map skipped input positions to the previous output position. */
+        	if (outputPositions != NULL)
+        	  {
+        	    int tcc;
+        	    for (tcc = 0; tcc < transCharslen; tcc++)
+        	      outputPositions[prevSrcMapping[src + tcc]] = dest - 1;
+        	  }
+        	if (!cursorStatus && src <= cursorPosition
+        	    && cursorPosition < src + transCharslen)
+        	  {
+        	    cursorStatus = 1;
+        	    cursorPosition = dest - 1;
+        	  }
+        	src += repwordLength + transCharslen;
+              }
+            src -= transCharslen;
+            break;
+          }
+        case CTO_JoinNum:
+        case CTO_JoinableWord:
+          while (src < srcmax
+        	 && checkAttr (currentInput[src], CTC_Space, 0) &&
+        	 currentInput[src] != ENDSEGMENT)
+            src++;
+          break;
+        default:
+          break;
+        }
       if (((src > 0) && checkAttr (currentInput[src - 1], CTC_Space, 0)
-	   && (transOpcode != CTO_JoinableWord)))
-	{
-	  srcword = src;
-	  destword = dest;
-	}
+           && (transOpcode != CTO_JoinableWord)))
+        {
+          srcword = src;
+          destword = dest;
+        }
       if (srcSpacing != NULL && srcSpacing[src] >= '0' && srcSpacing[src] <=
-	  '9')
-	destSpacing[dest] = srcSpacing[src];
+          '9')
+        destSpacing[dest] = srcSpacing[src];
       if ((transOpcode >= CTO_Always && transOpcode <= CTO_None) ||
-	  (transOpcode >= CTO_Digit && transOpcode <= CTO_LitDigit))
-	prevTransOpcode = transOpcode;
-    }				/*end of translation loop */
-  if (haveEmphasis && !wordsMarked && prevPrevType != plain_text)
+          (transOpcode >= CTO_Digit && transOpcode <= CTO_LitDigit))
+        prevTransOpcode = transOpcode;
+    }        			/*end of translation loop */
+  if (haveEmphasis && !wordsMarked && prevTypeform != plain_text
+      && src > 2 && typebuf[src - 1] == typebuf[src - 2])
     insertBrailleIndicators (2);
 failure:
   if (destword != 0 && src < srcmax
@@ -2057,12 +2074,12 @@ failure:
   if (src < srcmax)
     {
       while (checkAttr (currentInput[src], CTC_Space, 0))
-	if (++src == srcmax)
-	  break;
+        if (++src == srcmax)
+          break;
     }
   realInlen = src;
   return 1;
-}				/*first pass translation completed */
+}        			/*first pass translation completed */
 
 int EXPORT_CALL
 lou_hyphenate (const char *tableList, const widechar
