@@ -1,5 +1,4 @@
-/* liblouis Braille Translation and Back-Translation 
-Library
+/* liblouis Braille Translation and Back-Translation Library
 
    Based on the Linux screenreader BRLTTY, copyright (C) 1999-2006 by
    The BRLTTY Team
@@ -26,7 +25,6 @@ Library
    the Free Software Foundation, 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 
-   Maintained by John J. Boyer john.boyer@jjb-software.com
    */
 
 #ifndef __LOUIS_H_
@@ -71,7 +69,7 @@ extern "C"
 /*HASHNUM must be prime */
 #define HASHNUM 1123
 
-#define MAXSTRING 512
+#define MAXSTRING 2048
 
   typedef unsigned int TranslationTableOffset;
 #define OFFSETSIZE sizeof (TranslationTableOffset)
@@ -477,10 +475,17 @@ extern "C"
     alloc_srcMapping,
     alloc_prevSrcMapping
   } AllocBuf;
+
 /* The following function definitions are hooks into 
 * compileTranslationTable.c. Some are used by other library modules. 
 * Others are used by tools like lou_allround.c and lou_debug.c. */
 
+  char * getTablePath();
+/* Comma separated list of directories to search for tables. */
+
+  char ** resolveTable(const char *tableList, const char *base);
+/* Resolve tableList against base. */
+
   widechar getDotsForChar (widechar c);
 /* Returns the single-cell dot pattern corresponding to a character. */
 
@@ -533,14 +538,16 @@ extern "C"
   int other_translate (const char *trantab, const widechar
 		       * inbuf,
 		       int *inlen, widechar * outbuf, int *outlen,
-		       char *typeform, char *spacing, int *outputPos, int
+		       formtype *typeform, char *spacing, int 
+		       *outputPos, int
 		       *inputPos, int *cursorPos, int mode);
 
 /*Call wrappers for other translators */
   int other_backTranslate (const char *trantab, const widechar
 			   * inbuf,
 			   int *inlen, widechar * outbuf, int *outlen,
-			   char *typeform, char *spacing, int *outputPos, int
+			   formtype *typeform, char *spacing, int 
+			   *outputPos, int
 			   *inputPos, int *cursorPos, int mode);
 /*Call wrappers for other back-translators.*/
 
@@ -551,7 +558,8 @@ extern "C"
 
   int trace_translate (const char* tableList, const widechar* inbuf,
                        int* inlen, widechar* outbuf, int* outlen,
-                       char* typeform, char* spacing, int* outputPos,
+                       formtype* typeform, char* spacing, int* 
+                       outputPos,
                        int* inputPos, int* cursorPos,
                        const TranslationTableRule** rules, int* rulesLen,
                        int mode);
@@ -562,6 +570,12 @@ extern "C"
 void outOfMemory ();
 /* Priknts an out-of-memory message and exits*/
 
+void logWidecharBuf(logLevels level, const char *msg, const widechar *wbuf, int wlen);
+/* Helper for logging a widechar buffer */
+
+void logMessage(logLevels level, const char *format, ...);
+void closeLogFile();
+/* Function for closing loggin file */
 #ifdef __cplusplus
 }
 #endif				/* __cplusplus */
