@@ -1,3 +1,12 @@
+/* liblouis Braille Translation and Back-Translation Library
+
+Copyright (C) 2013,2015 Swiss Library for the Blind, Visually Impaired and Print Disabled
+
+Copying and distribution of this file, with or without modification,
+are permitted in any medium without royalty provided the copyright
+notice and this notice are preserved. This file is offered as-is,
+without any warranty. */
+
 #include <stdio.h>
 #include "brl_checks.h"
 #include "liblouis.h"
@@ -21,9 +30,9 @@ main (int argc, char **argv)
   }
 
   result |= check_translation(table, "aaaaa", NULL, "a");
-  /* Strangely enough subsequent translations fail */
   result |= check_translation(table, "aaaaa", NULL, "a");
   result |= check_translation(table, "aazzz", NULL, "a");
 
+  lou_free();
   return result;
 }
