@@ -1,12 +1,13 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <assert.h>
+#include "liblouis.h"
 #include "brl_checks.h"
 
 int main(int argc, char **argv)
 {
   int ret = 0;
-  char *tables = "da-dk-g2.ctb";
+  char *tables = "da-dk-g26.ctb";
   char *word = "alderen";
   char * hyphens = calloc(9, sizeof(char));
 
@@ -22,6 +23,7 @@ int main(int argc, char **argv)
   assert(hyphens[7] == '\0');
   assert(hyphens[8] == '\0');
   free(hyphens);
+  lou_free();
   return ret;
 
 }
