@@ -1,6 +1,16 @@
+/* liblouis Braille Translation and Back-Translation Library
+
+Copyright (C) 2012 Swiss Library for the Blind, Visually Impaired and Print Disabled
+
+Copying and distribution of this file, with or without modification,
+are permitted in any medium without royalty provided the copyright
+notice and this notice are preserved. This file is offered as-is,
+without any warranty. */
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include "liblouis.h"
 #include "brl_checks.h"
 
 int
@@ -48,6 +58,8 @@ main(int argc, char **argv)
   result |= check_translation(table3, txt, NULL, brl3);
   result |= check_translation(table4, txt, NULL, brl4);
 
+  lou_free();
+
   return result;
 
 }
