@@ -10,6 +10,7 @@ without any warranty. */
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include "liblouis.h"
 #include "brl_checks.h"
 
 int
@@ -32,5 +33,7 @@ main(int argc, char **argv)
 
   result |= check_translation("de-ch-g1.ctb", str, typeform, expected);
 
+  lou_free();
+
   return result;
 }
