@@ -1,6 +1,16 @@
+/* liblouis Braille Translation and Back-Translation Library
+
+Copyright (C) 2012 Swiss Library for the Blind, Visually Impaired and Print Disabled
+
+Copying and distribution of this file, with or without modification,
+are permitted in any medium without royalty provided the copyright
+notice and this notice are preserved. This file is offered as-is,
+without any warranty. */
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include "liblouis.h"
 #include "brl_checks.h"
 
 int
@@ -31,6 +41,8 @@ main(int argc, char **argv)
 
   result |= check_inpos(table, str3, expected_inpos3);
 
+  lou_free();
+
   return result;
 
 }
