@@ -1,3 +1,12 @@
+/* liblouis Braille Translation and Back-Translation Library
+
+Copyright (C) 2012 Swiss Library for the Blind, Visually Impaired and Print Disabled
+
+Copying and distribution of this file, with or without modification,
+are permitted in any medium without royalty provided the copyright
+notice and this notice are preserved. This file is offered as-is,
+without any warranty. */
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -21,5 +30,7 @@ main(int argc, char **argv)
   result |= check_translation("lowercase_with_unicode.ctb", str1, NULL, expected);
   result |= check_translation("lowercase_with_unicode.ctb", str2, NULL, expected);
 
+  lou_free();
+
   return result;
 }
