@@ -20,13 +20,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-   Maintained by John J. Boyer john.boyer@jjb-software.com
    */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
+# include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -52,7 +48,7 @@ static void
 print_help (void)
 {
   printf ("\
-Usage: %s [OPTION]\n", program_name);
+Usage: %s [OPTIONS]\n", program_name);
   
   fputs ("\
 Check the accuracy of hyphenation in Braille translation for both\n\
@@ -63,8 +59,14 @@ translated and untranslated words.\n\n", stdout);
   -v, --version       display version information and exit\n", stdout);
 
   printf ("\n");
-  printf ("\
-Report bugs to <%s>.\n", PACKAGE_BUGREPORT);
+  printf ("Report bugs to %s.\n", PACKAGE_BUGREPORT);
+
+#ifdef PACKAGE_PACKAGER_BUG_REPORTS
+  printf ("Report %s bugs to: %s\n", PACKAGE_PACKAGER, PACKAGE_PACKAGER_BUG_REPORTS);
+#endif
+#ifdef PACKAGE_URL
+  printf ("%s home page: <%s>\n", PACKAGE_NAME, PACKAGE_URL);
+#endif
 }
 
 #define BUFSIZE 256
