@@ -20,13 +20,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-   Maintained by John J. Boyer john.boyer@jjb-software.com
    */
 
-#ifdef HAVE_CONFIG_H
-# include "config.h"
-#endif
-
+# include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
