@@ -17,7 +17,7 @@
 
    */
 
-#include "config.h"
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
